import "leaflet/dist/leaflet.css"
import L from 'leaflet'
import {MapContainer, Marker, TileLayer, useMapEvents} from 'react-leaflet'
import {useEffect, useMemo, useState} from "react"
import dayjs from "dayjs"
import polyline from "@mapbox/polyline"
import getDistance from '@turf/rhumb-distance'
import getBearing from '@turf/rhumb-bearing'

export default function TrainMap () {
  return <>
    <MapContainer center={[46.47, 2.37]} zoom={6} style={{height: "100vh"}}>
      <TileLayer
        attribution='Données cartographiques : &copy; Les contributeurices <a href="https://www.openstreetmap.org/copyright">OpenStreetMap</a>'
        url="https://{s}.tile.openstreetmap.org/{z}/{x}/{y}.png"
      />
      <TileLayer
        attribution="Rendu : OpenRailwayMap"
        url="https://{s}.tiles.openrailwaymap.org/standard/{z}/{x}/{y}.png"></TileLayer>

      <MapContent />
    </MapContainer>
  </>
}

function MapContent () {
  const [latitude, setLatitude] = useState(46.47)
  const [longitude, setLongitude] = useState(2.37)
  const [zoom, setZoom] = useState(6)

  useEffect(() => {
    fetch(`${process.env.REACT_APP_MOTIS_SERVER}/api/v1/map/initial`)
      .then(response => response.json())
      .then(data => {
        setLatitude(data['lat'])
        setLongitude(data['lon'])
        setZoom(data['zoom'])
      })
  }, [])

  const map = useMapEvents({
    moveend: () => {
      updateTrips(map, setTrips)
    },
    zoomend: () => {
      updateTrips(map, setTrips)
    }
  })

  useEffect(() => {
    map.flyTo([latitude, longitude], zoom)
  }, [map, latitude, longitude, zoom])

  const [trips, setTrips] = useState([])

  useEffect(() => {
    updateTrips(map, setTrips)
    setInterval(() => updateTrips(map, setTrips), 30000)
  }, [map])

  return <>
    {trips.map(trip => <TripMarker trip={trip} />)}
  </>
}

function TripMarker ({trip}) {
  const [position, setPosition] = useState([trip.from.lat, trip.from.lon])
  const [heading, setHeading] = useState(0)
  const style = getModeStyle(trip.mode)

  const keyframes = useMemo(() => {
    const keyframes = []
    const departure = dayjs(trip.departure)
    const arrival = dayjs(trip.arrival)
    const coordinates = polyline.decode(trip.polyline)
    const totalDuration = arrival.diff(departure, 'seconds')
    let currDistance = 0

    let totalDistance = 0
		for (let i = 0; i < coordinates.length - 1; i++) {
			let from = coordinates[i]
			let to = coordinates[i + 1]
			totalDistance += getDistance(from, to, { units: 'meters' })
		}

		for (let i = 0; i < coordinates.length - 1; i++) {
			let from = coordinates[i]
			let to = coordinates[i + 1]

			const distance = getDistance(from, to, { units: 'meters' })
			const heading = getBearing(from, to)

			const r = currDistance / totalDistance
			keyframes.push({ point: from, time: departure.add(r * totalDuration, 'seconds'), heading: heading })

			currDistance += distance
		}
		keyframes.push({ point: coordinates[coordinates.length - 1], time: arrival, heading: 0 })

    return keyframes
  }, [trip])

  useEffect(() => {
    const interval = setInterval(() => {
      const now = dayjs()
      const index = keyframes.findIndex((kf) => kf.time >= now)
      if (index === -1 || index === 0)
        return

      const startState = keyframes[index - 1]
		  const endState = keyframes[index]
      const r = (now.diff(startState.time)) / (endState.time.diff(startState.time))
      const lat = startState.point[0] * (1 - r) + endState.point[0] * r
      const lon = startState.point[1] * (1 - r) + endState.point[1] * r
      setPosition([lat, lon])
      setHeading(startState.heading)
    }, 100)
    return () => clearInterval(interval)
  }, [keyframes])
  const icon = L.divIcon({
    html: `<svg fill="${style[1]}" fill-opacity="0.8" xmlns="http://www.w3.org/2000/svg"
\t width="36px" height="36px" viewBox="0 0 512 512" xml:space="preserve">
<g transform="rotate(${-heading - 90}, 256, 256)">
\t<path d="M256 17.108c-75.73 0-137.122 61.392-137.122 137.122.055 23.25 6.022 46.107 11.58 56.262L256 494.892l119.982-274.244h-.063c11.27-20.324 17.188-43.18 17.202-66.418C393.122 78.5 331.73 17.108 256 17.108zm0 68.56a68.56 68.56 0 0 1 68.56 68.562A68.56 68.56 0 0 1 256 222.79a68.56 68.56 0 0 1-68.56-68.56A68.56 68.56 0 0 1 256 85.67z" />
</g>
</svg>`,
    className: "",
    iconSize: [36, 36],
    iconAnchor: [36, 36],
  })
  return <Marker position={position} icon={icon} />
}

function getModeStyle (mode) {
	switch (mode) {
		case 'WALK':
		case 'FLEXIBLE':
			return ['walk', 'hsl(var(--foreground) / 1)', 'hsl(var(--background) / 1)']
		case 'BIKE':
		case 'BIKE_TO_PARK':
		case 'BIKE_RENTAL':
		case 'SCOOTER_RENTAL':
			return ['bike', '#075985', 'white']
		case 'CAR':
		case 'CAR_TO_PARK':
		case 'CAR_HAILING':
		case 'CAR_SHARING':
		case 'CAR_PICKUP':
		case 'CAR_RENTAL':
			return ['car', '#333', 'white']
		case 'TRANSIT':
		case 'BUS':
			return ['bus', '#ff9800', 'white']
		case 'COACH':
			return ['bus', '#9ccc65', 'white']
		case 'TRAM':
			return ['tram', '#ff9800', 'white']
		case 'METRO':
			return ['sbahn', '#4caf50', 'white']
		case 'SUBWAY':
			return ['ubahn', '#3f51b5', 'white']
		case 'FERRY':
			return ['ship', '#00acc1', 'white']
		case 'AIRPLANE':
			return ['plane', '#90a4ae', 'white']
		case 'HIGHSPEED_RAIL':
			return ['train', '#9c27b0', 'white']
		case 'LONG_DISTANCE':
			return ['train', '#e91e63', 'white']
		case 'NIGHT_RAIL':
			return ['train', '#1a237e', 'white']
		case 'REGIONAL_FAST_RAIL':
		case 'REGIONAL_RAIL':
		case 'RAIL':
			return ['train', '#f44336', 'white']
	}
	return ['train', '#000000', 'white']
}

function updateTrips(map, setTrips) {
  const bounds = map.getBounds()
  const now = dayjs()
  const now_plus_1_min = now.add(60000)
  const query_params = new URLSearchParams({
    min: `${bounds.getNorth()},${bounds.getWest()}`,
    max: `${bounds.getSouth()},${bounds.getEast()}`,
    zoom: map.getZoom(),
    startTime: now.format(),
    endTime: now_plus_1_min.format(),
  }).toString()
  fetch(`${process.env.REACT_APP_MOTIS_SERVER}/api/v1/map/trips?${query_params}`)
    .then(data => data.json())
    .then(setTrips)
}
